@@ -5,26 +5,28 @@
 {-# LANGUAGE CPP               #-}
 module Yesod.Core.Class.Yesod where
 
-import           Control.Monad.Logger               (logErrorS)
 import           Yesod.Core.Content
 import           Yesod.Core.Handler
 
 import           Yesod.Routes.Class
 
-import           Blaze.ByteString.Builder           (Builder)
-import           Blaze.ByteString.Builder.Char.Utf8 (fromText)
+import           Blaze.ByteString.Builder           (Builder, toByteString)
+import           Blaze.ByteString.Builder.ByteString (copyByteString)
+import           Blaze.ByteString.Builder.Char.Utf8 (fromText, fromChar)
 import           Control.Arrow                      ((***), second)
 import           Control.Exception                  (bracket)
+#if __GLASGOW_HASKELL__ < 710
+import           Control.Applicative                ((<$>))
+#endif
 import           Control.Monad                      (forM, when, void)
 import           Control.Monad.IO.Class             (MonadIO (liftIO))
 import           Control.Monad.Logger               (LogLevel (LevelInfo, LevelOther),
-                                                     LogSource)
+                                                     LogSource, logErrorS)
 import           Control.Monad.Trans.Resource       (InternalState, createInternalState, closeInternalState)
 import qualified Data.ByteString.Char8              as S8
 import qualified Data.ByteString.Lazy               as L
 import Data.Aeson (object, (.=))
-import           Data.List                          (foldl')
-import           Data.List                          (nub)
+import           Data.List                          (foldl', nub)
 import qualified Data.Map                           as Map
 import           Data.Monoid
 import           Data.Text                          (Text)
@@ -35,7 +37,7 @@ import           Data.Text.Lazy.Builder             (toLazyText)
 import           Data.Text.Lazy.Encoding            (encodeUtf8)
 import           Data.Word                          (Word64)
 import           Language.Haskell.TH.Syntax         (Loc (..))
-import           Network.HTTP.Types                 (encodePath)
+import           Network.HTTP.Types                 (encodePath, renderQueryText)
 import qualified Network.Wai                        as W
 import           Data.Default                       (def)
 import           Network.Wai.Parse                  (lbsBackEnd,
@@ -43,14 +45,14 @@ import           Network.Wai.Parse                  (lbsBackEnd,
 import           Network.Wai.Logger                 (ZonedDate, clockDateCacher)
 import           System.Log.FastLogger
 import           Text.Blaze                         (customAttribute, textTag,
-                                                     toValue, (!))
-import           Text.Blaze                         (preEscapedToMarkup)
+                                                     toValue, (!),
+                                                     preEscapedToMarkup)
 import qualified Text.Blaze.Html5                   as TBH
 import           Text.Hamlet
 import           Text.Julius
 import qualified Web.ClientSession                  as CS
-import           Web.Cookie                         (parseCookies)
-import           Web.Cookie                         (SetCookie (..))
+import           Web.Cookie                         (SetCookie (..), parseCookies, sameSiteLax,
+                                                     sameSiteStrict, SameSiteOption)
 import           Yesod.Core.Types
 import           Yesod.Core.Internal.Session
 import           Yesod.Core.Widget
@@ -107,6 +109,28 @@ class RenderRoute site => Yesod site where
     urlRenderOverride :: site -> Route site -> Maybe Builder
     urlRenderOverride _ _ = Nothing
 
+    -- | Override the rendering function for a particular URL and query string
+    -- parameters. One use case for this is to offload static hosting to a
+    -- different domain name to avoid sending cookies.
+    -- 
+    -- For backward compatibility default implementation is in terms of
+    -- 'urlRenderOverride', probably ineffective
+    -- 
+    -- Since 1.4.23
+    urlParamRenderOverride :: site
+                           -> Route site
+                           -> [(T.Text, T.Text)] -- ^ query string
+                           -> Maybe Builder
+    urlParamRenderOverride y route params = addParams params <$> urlRenderOverride y route
+      where
+        addParams [] routeBldr = routeBldr
+        addParams nonEmptyParams routeBldr =
+            let routeBS = toByteString routeBldr
+                qsSeparator = fromChar $ if S8.elem '?' routeBS then '&' else '?'
+                valueToMaybe t = if t == "" then Nothing else Just t
+                queryText = map (id *** valueToMaybe) nonEmptyParams
+            in copyByteString routeBS `mappend` qsSeparator `mappend` renderQueryText False queryText
+
     -- | Determine if a request is authorized or not.
     --
     -- Return 'Authorized' if the request is authorized,
@@ -237,7 +261,7 @@ class RenderRoute site => Yesod site where
     --
     -- Default: Uses clientsession with a 2 hour timeout.
     makeSessionBackend :: site -> IO (Maybe SessionBackend)
-    makeSessionBackend _ = fmap Just $ defaultClientSessionBackend 120 CS.defaultKeyFile
+    makeSessionBackend _ = Just <$> defaultClientSessionBackend 120 CS.defaultKeyFile
 
     -- | How to store uploaded files.
     --
@@ -290,6 +314,7 @@ class RenderRoute site => Yesod site where
     yesodWithInternalState :: site -> Maybe (Route site) -> (InternalState -> IO a) -> IO a
     yesodWithInternalState _ _ = bracket createInternalState closeInternalState
     {-# INLINE yesodWithInternalState #-}
+{-# DEPRECATED urlRenderOverride "Use urlParamRenderOverride instead" #-}
 
 -- | Default implementation of 'makeLogger'. Sends to stdout and
 -- automatically flushes on each write.
@@ -362,6 +387,34 @@ sslOnlySessions = (fmap . fmap) secureSessionCookies
     setSecureBit cookie = cookie { setCookieSecure = True }
     secureSessionCookies = customizeSessionCookies setSecureBit
 
+-- | Helps defend against CSRF attacks by setting the SameSite attribute on
+-- session cookies to Lax. With the Lax setting, the cookie will be sent with same-site
+-- requests, and with cross-site top-level navigations.
+--
+-- This option is liable to change in future versions of Yesod as the spec evolves.
+-- View more information <https://datatracker.ietf.org/doc/draft-west-first-party-cookies/ here>.
+--
+-- @since 1.4.23
+laxSameSiteSessions :: IO (Maybe SessionBackend) -> IO (Maybe SessionBackend)
+laxSameSiteSessions = sameSiteSession sameSiteLax
+
+-- | Helps defend against CSRF attacks by setting the SameSite attribute on
+-- session cookies to Strict. With the Strict setting, the cookie will only be
+-- sent with same-site requests.
+--
+-- This option is liable to change in future versions of Yesod as the spec evolves.
+-- View more information <https://datatracker.ietf.org/doc/draft-west-first-party-cookies/ here>.
+--
+-- @since 1.4.23
+strictSameSiteSessions :: IO (Maybe SessionBackend) -> IO (Maybe SessionBackend)
+strictSameSiteSessions = sameSiteSession sameSiteStrict
+
+sameSiteSession :: SameSiteOption -> IO (Maybe SessionBackend) -> IO (Maybe SessionBackend)
+sameSiteSession s = (fmap . fmap) secureSessionCookies
+  where
+    sameSite cookie = cookie { setCookieSameSite = Just s }
+    secureSessionCookies = customizeSessionCookies sameSite
+
 -- | Apply a Strict-Transport-Security header with the specified timeout to
 -- all responses so that browsers will rewrite all http links to https
 -- until the timeout expires. For security, the max-age of the STS header
@@ -388,8 +441,7 @@ sslOnlyMiddleware timeout handler = do
 --
 -- Since 1.2.0
 authorizationCheck :: Yesod site => HandlerT site IO ()
-authorizationCheck = do
-    getCurrentRoute >>= maybe (return ()) checkUrl
+authorizationCheck = getCurrentRoute >>= maybe (return ()) checkUrl
   where
     checkUrl url = do
         isWrite <- isWriteRequest url
@@ -399,21 +451,21 @@ authorizationCheck = do
             AuthenticationRequired -> do
                 master <- getYesod
                 case authRoute master of
-                    Nothing -> void $ notAuthenticated
-                    Just url' -> do
+                    Nothing -> void notAuthenticated
+                    Just url' ->
                       void $ selectRep $ do
                           provideRepType typeHtml $ do
                               setUltDestCurrent
                               void $ redirect url'
                           provideRepType typeJson $
-                              void $ notAuthenticated
+                              void notAuthenticated
             Unauthorized s' -> permissionDenied s'
 
 -- | Calls 'csrfCheckMiddleware' with 'isWriteRequest', 'defaultCsrfHeaderName', and 'defaultCsrfParamName' as parameters.
 --
 -- Since 1.4.14
 defaultCsrfCheckMiddleware :: Yesod site => HandlerT site IO res -> HandlerT site IO res
-defaultCsrfCheckMiddleware handler = do
+defaultCsrfCheckMiddleware handler =
     csrfCheckMiddleware
         handler
         (getCurrentRoute >>= maybe (return False) isWriteRequest)
@@ -455,6 +507,18 @@ csrfSetCookieMiddleware handler cookie = setCsrfCookieWithCookie cookie >> handl
 --
 -- For details, see the "AJAX CSRF protection" section of "Yesod.Core.Handler".
 --
+-- You can add this chain this middleware together with other middleware like so:
+--
+-- @
+-- 'yesodMiddleware' = 'defaultYesodMiddleware' . 'defaultCsrfMiddleware'
+-- @
+--
+-- or:
+--
+-- @
+-- 'yesodMiddleware' app = 'defaultYesodMiddleware' $ 'defaultCsrfMiddleware' $ app
+-- @
+--
 -- Since 1.4.14
 defaultCsrfMiddleware :: Yesod site => HandlerT site IO res -> HandlerT site IO res
 defaultCsrfMiddleware = defaultCsrfSetCookieMiddleware . defaultCsrfCheckMiddleware
@@ -592,12 +656,9 @@ defaultErrorHandler NotAuthenticated = selectRep $ do
         -- The client will just use the authentication_url in the JSON
         site <- getYesod
         rend <- getUrlRender
-        return $ object $ [
-          "message" .= ("Not logged in"::Text)
-          ] ++
-          case authRoute site of
-              Nothing -> []
-              Just url -> ["authentication_url" .= rend url]
+        let apair u = ["authentication_url" .= rend u]
+            content = maybe [] apair (authRoute site)
+        return $ object $ ("message" .= ("Not logged in"::Text)):content
 
 defaultErrorHandler (PermissionDenied msg) = selectRep $ do
     provideRep $ defaultLayout $ do
@@ -607,9 +668,7 @@ defaultErrorHandler (PermissionDenied msg) = selectRep $ do
             <p>#{msg}
         |]
     provideRep $
-        return $ object $ [
-          "message" .= ("Permission Denied. " <> msg)
-          ]
+        return $ object ["message" .= ("Permission Denied. " <> msg)]
 
 defaultErrorHandler (InvalidArgs ia) = selectRep $ do
     provideRep $ defaultLayout $ do
@@ -641,8 +700,8 @@ defaultErrorHandler (BadMethod m) = selectRep $ do
     provideRep $ return $ object ["message" .= ("Bad method" :: Text), "method" .= TE.decodeUtf8With TEE.lenientDecode m]
 
 asyncHelper :: (url -> [x] -> Text)
-         -> [Script (url)]
-         -> Maybe (JavascriptUrl (url))
+         -> [Script url]
+         -> Maybe (JavascriptUrl url)
          -> Maybe Text
          -> (Maybe (HtmlUrl url), [Text])
 asyncHelper render scripts jscript jsLoc =
@@ -732,8 +791,7 @@ defaultClientSessionBackend :: Int -- ^ minutes
                             -> IO SessionBackend
 defaultClientSessionBackend minutes fp = do
   key <- CS.getKey fp
-  let timeout = fromIntegral (minutes * 60)
-  (getCachedDate, _closeDateCacher) <- clientSessionDateCacher timeout
+  (getCachedDate, _closeDateCacher) <- clientSessionDateCacher (minToSec minutes)
   return $ clientSessionBackend key getCachedDate
 
 -- | Create a @SessionBackend@ which reads the session key from the named
@@ -759,10 +817,12 @@ envClientSessionBackend :: Int -- ^ minutes
                         -> IO SessionBackend
 envClientSessionBackend minutes name = do
     key <- CS.getKeyEnv name
-    let timeout = fromIntegral (minutes * 60)
-    (getCachedDate, _closeDateCacher) <- clientSessionDateCacher timeout
+    (getCachedDate, _closeDateCacher) <- clientSessionDateCacher $ minToSec minutes
     return $ clientSessionBackend key getCachedDate
 
+minToSec :: (Integral a, Num b) => a -> b
+minToSec minutes = fromIntegral (minutes * 60)
+
 jsToHtml :: Javascript -> Html
 jsToHtml (Javascript b) = preEscapedToMarkup $ toLazyText b
 
@@ -818,8 +878,14 @@ loadClientSession key getCachedDate sessionName req = load
 -- turn the TH Loc loaction information into a human readable string
 -- leaving out the loc_end parameter
 fileLocationToString :: Loc -> String
-fileLocationToString loc = (loc_package loc) ++ ':' : (loc_module loc) ++
-  ' ' : (loc_filename loc) ++ ':' : (line loc) ++ ':' : (char loc)
+fileLocationToString loc =
+    concat
+      [ loc_package loc
+      , ':' : loc_module loc
+      , ' ' : loc_filename loc
+      , ':' : line loc
+      , ':' : char loc
+      ]
   where
     line = show . fst . loc_start
     char = show . snd . loc_start
