@@ -6,6 +6,9 @@ module Yesod.Core.Json
       defaultLayoutJson
     , jsonToRepJson
     , returnJson
+#if MIN_VERSION_aeson(0, 11, 0)
+    , returnJsonEncoding
+#endif
     , provideJson
 
       -- * Convert to a JSON value
@@ -24,6 +27,9 @@ module Yesod.Core.Json
 
       -- * Convenience functions
     , jsonOrRedirect
+#if MIN_VERSION_aeson(0, 11, 0)
+    , jsonEncodingOrRedirect
+#endif
     , acceptsJson
     ) where
 
@@ -52,35 +58,51 @@ import Control.Monad (liftM)
 -- data, using the default layout for the HTML output
 -- ('defaultLayout').
 --
--- /Since: 0.3.0/
+-- @since 0.3.0
 defaultLayoutJson :: (Yesod site, J.ToJSON a)
                   => WidgetT site IO ()  -- ^ HTML
                   -> HandlerT site IO a  -- ^ JSON
                   -> HandlerT site IO TypedContent
 defaultLayoutJson w json = selectRep $ do
     provideRep $ defaultLayout w
+#if MIN_VERSION_aeson(0, 11, 0)
+    provideRep $ fmap J.toEncoding json
+#else
     provideRep $ fmap J.toJSON json
+#endif
 
 -- | Wraps a data type in a 'RepJson'.  The data type must
 -- support conversion to JSON via 'J.ToJSON'.
 --
--- /Since: 0.3.0/
+-- @since 0.3.0
 jsonToRepJson :: (Monad m, J.ToJSON a) => a -> m J.Value
 jsonToRepJson = return . J.toJSON
 {-# DEPRECATED jsonToRepJson "Use returnJson instead" #-}
 
 -- | Convert a value to a JSON representation via aeson\'s 'J.toJSON' function.
 --
--- Since 1.2.1
+-- @since 1.2.1
 returnJson :: (Monad m, J.ToJSON a) => a -> m J.Value
 returnJson = return . J.toJSON
 
--- | Provide a JSON representation for usage with 'selectReps', using aeson\'s
--- 'J.toJSON' function to perform the conversion.
+#if MIN_VERSION_aeson(0, 11, 0)
+-- | Convert a value to a JSON representation via aeson\'s 'J.toEncoding' function.
 --
--- Since 1.2.1
+-- @since 1.4.21
+returnJsonEncoding :: (Monad m, J.ToJSON a) => a -> m J.Encoding
+returnJsonEncoding = return . J.toEncoding
+#endif
+
+-- | Provide a JSON representation for usage with 'selectReps', using aeson\'s
+-- 'J.toJSON' (aeson >= 0.11: 'J.toEncoding') function to perform the conversion.
+--
+-- @since 1.2.1
 provideJson :: (Monad m, J.ToJSON a) => a -> Writer (Endo [ProvidedRep m]) ()
+#if MIN_VERSION_aeson(0, 11, 0)
+provideJson = provideRep . return . J.toEncoding
+#else
 provideJson = provideRep . return . J.toJSON
+#endif
 
 -- | Parse the request body to a data type as a JSON value.  The
 -- data type must support conversion from JSON via 'J.FromJSON'.
@@ -91,7 +113,7 @@ provideJson = provideRep . return . J.toJSON
 -- twice will result in a parse error on the second call, since the request
 -- body will no longer be available.
 --
--- /Since: 0.3.0/
+-- @since 0.3.0
 parseJsonBody :: (MonadHandler m, J.FromJSON a) => m (J.Result a)
 parseJsonBody = do
     eValue <- rawRequestBody $$ runCatchC (sinkParser JP.value')
@@ -129,9 +151,32 @@ jsonOrRedirect :: (MonadHandler m, J.ToJSON a)
                => Route (HandlerSite m) -- ^ Redirect target
                -> a            -- ^ Data to send via JSON
                -> m J.Value
-jsonOrRedirect r j = do
+jsonOrRedirect = jsonOrRedirect' J.toJSON
+
+#if MIN_VERSION_aeson(0, 11, 0)
+-- | jsonEncodingOrRedirect simplifies the scenario where a POST handler sends a different
+-- response based on Accept headers:
+--
+--     1. 200 with JSON data if the client prefers
+--     @application\/json@ (e.g. AJAX, see 'acceptsJSON').
+--
+--     2. 3xx otherwise, following the PRG pattern.
+-- @since 1.4.21
+jsonEncodingOrRedirect :: (MonadHandler m, J.ToJSON a)
+            => Route (HandlerSite m) -- ^ Redirect target
+            -> a            -- ^ Data to send via JSON
+            -> m J.Encoding
+jsonEncodingOrRedirect = jsonOrRedirect' J.toEncoding
+#endif
+
+jsonOrRedirect' :: (MonadHandler m, J.ToJSON a)
+            => (a -> b)
+            -> Route (HandlerSite m) -- ^ Redirect target
+            -> a            -- ^ Data to send via JSON
+            -> m b
+jsonOrRedirect' f r j = do
     q <- acceptsJson
-    if q then return (J.toJSON j)
+    if q then return (f j)
          else redirect r
 
 -- | Returns @True@ if the client prefers @application\/json@ as
